@@ -24,26 +24,60 @@ from mimerpy.mimPyExceptionHandler import *
 import mimerapi
 import collections
 
-get_funcs = {1: mimerapi.mimerGetString8, 2: mimerapi.mimerGetString8, 3: mimerapi.mimerGetString8, 6: mimerapi.mimerGetInt32,
-             10: mimerapi.mimerGetDouble,
-             11: mimerapi.mimerGetString8, 12: mimerapi.mimerGetString8, 13: mimerapi.mimerGetString8,
-             14: mimerapi.mimerGetString8, 34: mimerapi.mimerGetBinary, 35: mimerapi.mimerGetBinary,
-             39: mimerapi.mimerGetString8, 40: mimerapi.mimerGetString8,
-             42: mimerapi.mimerGetBoolean, 48: mimerapi.mimerGetInt32, 50: mimerapi.mimerGetInt32, 52: mimerapi.mimerGetInt64,
-             63: mimerapi.mimerGetString8,
-             56: mimerapi.mimerGetDouble, 54: mimerapi.mimerGetDouble, 57: mimerapi.mimerGetBlobData,
-             58: mimerapi.mimerGetNclobData8, 59: mimerapi.mimerGetNclobData8}
-set_funcs = {1: mimerapi.mimerSetString8, 2: mimerapi.mimerSetString8, 3: mimerapi.mimerSetString8, 6: mimerapi.mimerSetInt32,
-             10: mimerapi.mimerSetDouble,
-             11: mimerapi.mimerSetString8, 12: mimerapi.mimerSetString8, 13: mimerapi.mimerSetString8,
-             14: mimerapi.mimerSetString8, 34: mimerapi.mimerSetBinary, 35: mimerapi.mimerSetBinary,
-             39: mimerapi.mimerSetString8,
-             40: mimerapi.mimerSetString8, 42: mimerapi.mimerSetBoolean, 48: mimerapi.mimerSetInt32,
-             50: mimerapi.mimerSetInt32, 52: mimerapi.mimerSetInt64, 63: mimerapi.mimerSetString8, 56: mimerapi.mimerSetDouble,
-             54: mimerapi.mimerSetDouble,
-             57: mimerapi.mimerSetBlobData, 58: mimerapi.mimerSetNclobData8, 59: mimerapi.mimerSetNclobData8,
-             501: mimerapi.mimerSetNull}
+def _define_funcs():
+    global get_funcs
+    global set_funcs
 
+    get_funcs = {1: mimerapi.mimerGetString8,
+                 2: mimerapi.mimerGetString8,
+                 3: mimerapi.mimerGetString8,
+                 6: mimerapi.mimerGetInt32,
+                 10: mimerapi.mimerGetDouble,
+                 11: mimerapi.mimerGetString8,
+                 12: mimerapi.mimerGetString8,
+                 13: mimerapi.mimerGetString8,
+                 14: mimerapi.mimerGetString8,
+                 34: mimerapi.mimerGetBinary,
+                 35: mimerapi.mimerGetBinary,
+                 39: mimerapi.mimerGetString8,
+                 40: mimerapi.mimerGetString8,
+                 42: mimerapi.mimerGetBoolean,
+                 48: mimerapi.mimerGetInt32,
+                 50: mimerapi.mimerGetInt32,
+                 52: mimerapi.mimerGetInt64,
+                 63: mimerapi.mimerGetString8,
+                 56: mimerapi.mimerGetDouble,
+                 54: mimerapi.mimerGetDouble,
+                 57: mimerapi.mimerGetBlobData,
+                 58: mimerapi.mimerGetNclobData8,
+                 59: mimerapi.mimerGetNclobData8}
+
+    set_funcs = {1: mimerapi.mimerSetString8,
+                 2: mimerapi.mimerSetString8,
+                 3: mimerapi.mimerSetString8,
+                 6: mimerapi.mimerSetInt32,
+                 10: mimerapi.mimerSetDouble,
+                 11: mimerapi.mimerSetString8,
+                 12: mimerapi.mimerSetString8,
+                 13: mimerapi.mimerSetString8,
+                 14: mimerapi.mimerSetString8,
+                 34: mimerapi.mimerSetBinary,
+                 35: mimerapi.mimerSetBinary,
+                 39: mimerapi.mimerSetString8,
+                 40: mimerapi.mimerSetString8,
+                 42: mimerapi.mimerSetBoolean,
+                 48: mimerapi.mimerSetInt32,
+                 50: mimerapi.mimerSetInt32,
+                 52: mimerapi.mimerSetInt64,
+                 63: mimerapi.mimerSetString8,
+                 56: mimerapi.mimerSetDouble,
+                 54: mimerapi.mimerSetDouble,
+                 57: mimerapi.mimerSetBlobData,
+                 58: mimerapi.mimerSetNclobData8,
+                 59: mimerapi.mimerSetNclobData8,
+                 501: mimerapi.mimerSetNull}
+
+_define_funcs()
 
 class Cursor:
     """
@@ -147,7 +181,7 @@ class Cursor:
 
             # -24005 indicates a DDL statement
             if (self._DDL_rc_value != -24005):
-                self.__check_for_exception(rc_value, self.__session)
+                self.__check_mimerapi_error(rc_value, self.__session)
                 self.__statement = values[1]
 
         self._last_query = query
@@ -158,10 +192,10 @@ class Cursor:
             self.connection.transaction = False
             self.messages = []
             rc_value = mimerapi.mimerExecuteStatement8(self.__session, query)
-            self.__check_for_exception(rc_value, self.__session)
+            self.__check_mimerapi_error(rc_value, self.__session)
         else:
             rc_value = mimerapi.mimerParameterCount(self.__statement)
-            self.__check_for_exception(rc_value, self.__statement)
+            self.__check_mimerapi_error(rc_value, self.__statement)
 
             # Return value of mimerParameterCount = 0 implies a query with no
             # parameters.
@@ -170,52 +204,52 @@ class Cursor:
                 try:
 
                     if (len(parameter_markers) != self._number_of_parameters):
-                        self.__check_for_exception(-25103, "Invalid number of parameters")
+                        self.__raise_exception(-25013)
 
                     # Column number starts a 1
                     for cur_column in range(1, self._number_of_parameters + 1):
                         parameter_type = mimerapi.mimerParameterType(self.__statement, cur_column)
-                        self.__check_for_exception(parameter_type, self.__statement)
+                        self.__check_mimerapi_error(parameter_type, self.__statement)
 
                         if (isinstance(parameter_markers, dict)):
                             rc_value, parameter_name = mimerapi.mimerParameterName8(self.__statement, cur_column)
-                            self.__check_for_exception(rc_value, self.__statement)
+                            self.__check_mimerapi_error(rc_value, self.__statement)
                             if parameter_name in parameter_markers:
                                 parameter = parameter_markers.get(parameter_name)
                                 if (parameter == None):
                                     parameter_type = 501
                                 rc_value = set_funcs[parameter_type](self.__statement, cur_column, parameter)
                             else:
-                                self.__check_for_exception(-25108, "KeyError in parameters, key: " + str(parameter_name)
-                                                           + " does not exist in dictionary")
+                                self.__raise_exception(-25012,
+                                                       str(parameter_name))
                         else:
                             # If the parameter marker is None, we use mimerSetNull
                             if (parameter_markers[cur_column - 1] == None):
                                 parameter_type = 501
                             rc_value = set_funcs[parameter_type](self.__statement,
                                                                        cur_column, parameter_markers[cur_column - 1])
-                        self.__check_for_exception(rc_value, self.__statement)
+                        self.__check_mimerapi_error(rc_value, self.__statement)
 
                 # Catching error for errorhandler
                 except KeyError as e:
-                    self.__check_for_exception(-28100, e)
+                    self.__raise_exception(-25020, exception=e) # &&&& ??
                 # Catching error for errorhandler
                 except TypeError as e:
-                    self.__check_for_exception(-25107, e)
+                    self.__raise_exception(-25020, exception=e)
                 # Catching error for errorhandler
                 except OverflowError as e:
-                    self.__check_for_exception(-25107, e)
+                    self.__raise_exception(-25020, exception=e)
 
-            self.__check_for_exception(rc_value, self.__statement)
+            self.__check_mimerapi_error(rc_value, self.__statement)
             rc_value = mimerapi.mimerColumnCount(self.__statement)
 
             # Return value of mimerColumnCount <= 0 implies a query with no
             # result set.
             if (rc_value <= 0):
-                self.__check_for_exception(rc_value, self.__statement)
+                self.__check_mimerapi_error(rc_value, self.__statement)
                 self.messages = []
                 rc_value = mimerapi.mimerExecute(self.__statement)
-                self.__check_for_exception(rc_value, self.__statement)
+                self.__check_mimerapi_error(rc_value, self.__statement)
                 self.rowcount = rc_value
             else:
                 # Return value of mimerColumnCount > 0 implies a query with a
@@ -223,7 +257,7 @@ class Cursor:
                 self.rowcount = rc_value
                 self._number_of_columns = rc_value
                 rc_value = mimerapi.mimerOpenCursor(self.__statement)
-                self.__check_for_exception(rc_value, self.__statement)
+                self.__check_mimerapi_error(rc_value, self.__statement)
                 self.__mimcursor = True
                 description = collections.namedtuple('Column_description',
                                                      'name type_code display_size internal_size precision scale null_ok')
@@ -232,10 +266,10 @@ class Cursor:
                 for cur_column in range(1, self._number_of_columns + 1):
                     func_tuple = mimerapi.mimerColumnName8(self.__statement, cur_column)
                     rc_value = func_tuple[0]
-                    self.__check_for_exception(rc_value, self.__statement)
+                    self.__check_mimerapi_error(rc_value, self.__statement)
                     name = func_tuple[1]
                     rc_value = mimerapi.mimerColumnType(self.__statement, cur_column)
-                    self.__check_for_exception(rc_value, self.__statement)
+                    self.__check_mimerapi_error(rc_value, self.__statement)
                     type_code = rc_value
                     self.description = self.description + (description(name=name,
                                                                        type_code=type_code,
@@ -268,25 +302,24 @@ class Cursor:
         # I would like to look over this at some point
         # Checking for invalid parameter structure
         if (not isinstance(params, tuple) and not isinstance(params, list)):
-            self.__check_for_exception(-26001, "Invalid parameter format")
-
+            self.__raise_exception(-25013)
         else:
             if (not isinstance(params[0], tuple) and not isinstance(params[0], dict)):
-                self.__check_for_exception(-26001, "Invalid parameter format")
+                self.__raise_exception(-25013)
 
         self.__close_statement()
         values = mimerapi.mimerBeginStatement8(self.__session, query, 0)
         rc_value = values[0]
 
-        self.__check_for_exception(rc_value, self.__session)
+        self.__check_mimerapi_error(rc_value, self.__session)
         self.__statement = values[1]
-        self.__check_for_exception(rc_value, self.__statement)
+        self.__check_mimerapi_error(rc_value, self.__statement)
 
         rc_value = mimerapi.mimerParameterCount(self.__statement)
         self._number_of_parameters = rc_value
 
         self.rowcount = 0
-        self.__check_for_exception(rc_value, self.__statement)
+        self.__check_mimerapi_error(rc_value, self.__statement)
 
         try:
             for laps in range(0, len(params)):
@@ -295,19 +328,18 @@ class Cursor:
                 # Column number starts a 1
                 for cur_column in range(1, self._number_of_parameters + 1):
                     parameter_type = mimerapi.mimerParameterType(self.__statement, cur_column)
-                    self.__check_for_exception(parameter_type, self.__statement)
+                    self.__check_mimerapi_error(parameter_type, self.__statement)
 
                     if (isinstance(cur_param, dict)):
                         rc_value, parameter_name = mimerapi.mimerParameterName8(self.__statement, cur_column)
-                        self.__check_for_exception(rc_value, self.__statement)
+                        self.__check_mimerapi_error(rc_value, self.__statement)
                         if parameter_name in cur_param:
                             parameter = cur_param.get(parameter_name)
                             if (parameter == None):
                                 parameter_type = 501
                             rc_value = set_funcs[parameter_type](self.__statement, cur_column, parameter)
                         else:
-                            self.__check_for_exception(-25108, "KeyError in parameters, key: " + str(parameter_name)
-                                                       + ", does not exist in dictionary")
+                            self.__raise_exception(-25012, str(parameter_name))
                     else:
                         # If the parameter marker is None, we use mimerSetNull
                         if (cur_param[cur_column - 1] == None):
@@ -319,18 +351,18 @@ class Cursor:
                 # Batching after all parameters are set
                 if (laps != len(params) - 1):
                     rc_value = mimerapi.mimerAddBatch(self.__statement)
-                    self.__check_for_exception(rc_value, self.__statement)
+                    self.__check_mimerapi_error(rc_value, self.__statement)
                 self.rowcount = self.rowcount + rc_value
 
             rc_value = mimerapi.mimerExecute(self.__statement)
-            self.__check_for_exception(rc_value, self.__statement)
+            self.__check_mimerapi_error(rc_value, self.__statement)
 
         # Catching error for errorhandler
         except TypeError as e:
-            self.__check_for_exception(-25107, e)
+            self.__raise_exception(-25020, exception=e)
         # Catching error for errorhandler
         except OverflowError as e:
-            self.__check_for_exception(-25107, e)
+            self.__raise_exception(-25020, exception=e)
 
     def fetchone(self):
         """
@@ -346,10 +378,10 @@ class Cursor:
         self.__check_for_transaction()
 
         if (not self.__mimcursor):
-            self.__check_for_exception(-25107, "Previous execute did not produce any result set")
+            self.__raise_exception(-25014)
 
         rc_value = mimerapi.mimerFetch(self.__statement)
-        self.__check_for_exception(rc_value, self.__statement)
+        self.__check_mimerapi_error(rc_value, self.__statement)
         return_tuple = ()
 
         # Return value of mimerFetch == 100 implies end of result set
@@ -358,9 +390,9 @@ class Cursor:
 
         for cur_column in range(1, self._number_of_columns + 1):
             rc_value = mimerapi.mimerColumnType(self.__statement, cur_column)
-            self.__check_for_exception(rc_value, self.__statement)
+            self.__check_mimerapi_error(rc_value, self.__statement)
             func_tuple = get_funcs[rc_value](self.__statement, cur_column)
-            self.__check_for_exception(func_tuple[0], self.__statement)
+            self.__check_mimerapi_error(func_tuple[0], self.__statement)
 
             # Conversion from C int to Python boolean
             if (rc_value == 42 and not func_tuple[1] == None):
@@ -394,7 +426,7 @@ class Cursor:
         self.__check_for_transaction()
 
         if (not self.__mimcursor):
-            self.__check_for_exception(-25107, "Previous execute did not produce any result set")
+            self.__raise_exception(-25014)
 
         # If arg is provided, arraysize is set
         if (len(arg) > 0):
@@ -404,14 +436,14 @@ class Cursor:
         rc_value = mimerapi.mimerFetch(self.__statement)
         fetch_value = rc_value
         while (fetch_value != 100 and fetch_length > 0):
-            self.__check_for_exception(fetch_value, self.__statement)
+            self.__check_mimerapi_error(fetch_value, self.__statement)
             return_tuple = ()
             # Column number starts a 1
             for cur_column in range(1, self._number_of_columns + 1):
                 rc_value = mimerapi.mimerColumnType(self.__statement, cur_column)
-                self.__check_for_exception(rc_value, self.__statement)
+                self.__check_mimerapi_error(rc_value, self.__statement)
                 func_tuple = get_funcs[rc_value](self.__statement, cur_column)
-                self.__check_for_exception(func_tuple[0], self.__statement)
+                self.__check_mimerapi_error(func_tuple[0], self.__statement)
 
                 # Conversion from C int to Python boolean
                 if (rc_value == 42 and not func_tuple[1] == None):
@@ -440,19 +472,19 @@ class Cursor:
         self.__check_if_open()
         self.__check_for_transaction()
         if (not self.__mimcursor):
-            self.__check_for_exception(-25107, "Previous execute did not produce any result set")
+            self.__raise_exception(-25014)
         values = []
         rc_value = mimerapi.mimerFetch(self.__statement)
         fetch_value = rc_value
         while (fetch_value != 100):
-            self.__check_for_exception(fetch_value, self.__statement)
+            self.__check_mimerapi_error(fetch_value, self.__statement)
             return_tuple = ()
             # Column number starts a 1
             for cur_column in range(1, self._number_of_columns + 1):
                 rc_value = mimerapi.mimerColumnType(self.__statement, cur_column)
-                self.__check_for_exception(rc_value, self.__statement)
+                self.__check_mimerapi_error(rc_value, self.__statement)
                 func_tuple = get_funcs[rc_value](self.__statement, cur_column)
-                self.__check_for_exception(func_tuple[0], self.__statement)
+                self.__check_mimerapi_error(func_tuple[0], self.__statement)
 
                 # Conversion from C int to Python boolean
                 if (rc_value == 42 and not func_tuple[1] == None):
@@ -488,33 +520,43 @@ class Cursor:
 
     def __close_statement(self):
         # Private method for closing MimerStatement.
-        if (self.__statement != None):
+        if (self.__statement is not None and
+            self.connection._session is not None):
             rc_value = mimerapi.mimerEndStatement(self.__statement)
-            self.__statement = None
-            self.__mimcursor = False
-            self.__check_for_exception(rc_value, self.__statement)
+            self.__check_mimerapi_error(rc_value, self.__statement)
+        self.__statement = None
+        self.__mimcursor = False
 
     def __check_if_open(self):
         if (self.__session == None):
-            self.__check_for_exception(-25000, "Cursor not open")
+            self.__raise_exception(-25015)
 
     def __check_for_transaction(self):
         if (not self.connection._transaction and not self.connection.autocommitmode):
             rc_value = mimerapi.mimerBeginTransaction(self.__session)
-            self.__check_for_exception(rc_value, self.__session)
+            self.__check_mimerapi_error(rc_value, self.__session)
             self.connection._transaction = True
 
-    def __check_for_exception(self, *arg):
-        error_tuple = check_for_exception(arg[0], arg[1])
-        if (isinstance(error_tuple, tuple)):
-            if (error_tuple[0]):
-                self.errorhandler(None, self, error_tuple[0], (error_tuple[1]))
+    def __raise_exception(self, rc, val=None, exception=None):
+        msg = mimerpy_error[rc]
+        if val is not None:
+            msg = msg % val
+        if exception is None:
+            etup = (rc, msg)
+        else:
+            etup = (rc, msg, exception)
+        self.errorhandler(None, self, get_error_class(rc), etup)
+
+    def __check_mimerapi_error(self, rc, handle):
+        if rc < 0:
+            (ec, ev) = get_mimerapi_exception(rc, handle)
+            self.errorhandler(None, self, ec, ev)
 
     def nextset(self):
-        self.__check_for_exception(-28002, "Not suppported")
+        self.__raise_exception(-25000)
 
     def callproc(self):
-        self.__check_for_exception(-28003, "Not suppported")
+        self.__raise_exception(-25000)
 
 
 class ScrollCursor(Cursor):
@@ -563,7 +605,7 @@ class ScrollCursor(Cursor):
         self._Cursor__check_for_transaction()
 
         if (self.__result_set == None):
-            self._Cursor__check_for_exception(-25000, "Last execute did not produce a result set")
+            self._Cursor__raise_exception(-25014)
         values = ()
         try:
             values = values + self.__result_set[self.rownumber]
@@ -591,7 +633,7 @@ class ScrollCursor(Cursor):
         self._Cursor__check_for_transaction()
 
         if (self.__result_set == None):
-            self._Cursor__check_for_exception(-25000, "Last execute did not produce a result set")
+            self._Cursor__raise_exception(-25014)
 
         if (len(arg) > 0):
             self.arraysize = arg[0]
@@ -622,7 +664,7 @@ class ScrollCursor(Cursor):
         self._Cursor__check_for_transaction()
         values = []
         if (self.__result_set == None):
-            self._Cursor__check_for_exception(-25000, "Last execute did not produce a result set")
+            self._Cursor__raise_exception(-25014)
         if (not self.rownumber):
             self.rownumber = len(self.__result_set)
             return self.__result_set
@@ -642,7 +684,7 @@ class ScrollCursor(Cursor):
         self._Cursor__check_for_transaction()
 
         if (self.__result_set == None):
-            self._Cursor__check_for_exception(-25000, "Last execute did not produce a result set")
+            self._Cursor__raise_exception(-25014)
         if (self.__result_set == []):
             return self.__result_set
 
@@ -684,4 +726,4 @@ class ScrollCursor(Cursor):
             else:
                 self.rownumber = value
         else:
-            self._Cursor__check_for_exception(-25000, "Not a possible scroll mode")
+            self._Cursor__raise_exception(-25016)
