@@ -56,7 +56,7 @@ typedef long long pyptr;
  *  The size is large enough to be usable in most cases and small enough to not
  *  waste cycles filling a too small buffer.
  */
-#define BUFLEN 128
+#define BUFLEN 2
 
 
 
@@ -68,6 +68,30 @@ static const int lengthof_Terminating_NUL = 1;
 
 
 
+static PyObject* mimerAPIVersion(PyObject* self, PyObject* args)
+/* *********************************************************************/
+/**
+ *  @brief      Starts a session with the database.
+ *
+ *  @author     Erik Gunne & Magdalena Boström
+ *  @date       2017-06-16
+ *  @param[in]     self       Pointer to python object that function is
+ *                            called from.
+ *  @param[in]     args       Arguments. We don't read any...
+ *
+ *  @returns    Version string
+ */
+/* *********************************************************************/
+{
+    const char *version;
+
+    version = MimerAPIVersion();
+
+    return Py_BuildValue("s", version);
+}
+
+
+
 static PyObject* mimerBeginSession8(PyObject* self, PyObject* args)
 /* *********************************************************************/
 /**
@@ -529,6 +553,10 @@ static PyObject* mimerParameterName8(PyObject* self, PyObject* args)
         int buffer_size = rc + 1;
         PyObject* return_object;
         char *bigvalue = (char*) calloc(buffer_size, sizeof(char));
+        if(bigvalue == NULL) {
+                return(Py_BuildValue("i", 101));
+        }
+
         rc = MimerParameterName8((MimerStatement)statement, parameter_number,
                                  bigvalue, buffer_size);
         return_object = Py_BuildValue("is", rc, bigvalue);
@@ -668,6 +696,9 @@ static PyObject* mimerColumnName8(PyObject* self, PyObject* args)
         int buffer_size = rc + 1;
         PyObject* return_object;
         char *bigvalue = (char*) calloc(buffer_size, sizeof(char));
+        if(bigvalue == NULL) {
+            return(Py_BuildValue("i", 101));
+        }
         rc = MimerColumnName8((MimerStatement)statement, column_number,
                               bigvalue, buffer_size);
         return_object = Py_BuildValue("is", rc, bigvalue);
@@ -839,6 +870,9 @@ static PyObject* mimerGetString8(PyObject* self, PyObject* args)
         int buffer_size = rc + 1;
         PyObject* return_object;
         char *bigvalue = (char*) calloc(buffer_size, sizeof(char));
+        if(bigvalue == NULL) {
+            return(Py_BuildValue("i", 101));
+        }
         rc = MimerGetString8((MimerStatement)statement, column_number,
                              bigvalue, buffer_size);
         return_object = Py_BuildValue("is", rc, bigvalue);
@@ -1072,12 +1106,22 @@ static PyObject* mimerGetError8(PyObject* self, PyObject* args)
     }
 
     rc = MimerGetError8((MimerStatement)statement, &evalue, message, BUFLEN);
-    /* TODO &&&& What if BUFLEN is not enough? */
-    if (rc) {
-        /* No error returned */
-        message[0] = '\0';
+    
+    if (rc > BUFLEN) {
+        PyObject* return_object;
+        int big_buff = 512;
+        char *long_message;
+        long_message = (char*) calloc(big_buff, sizeof(char));
+        if(long_message == NULL) {
+            return(Py_BuildValue("i", 101));
+        }
+        rc = MimerGetError8((MimerStatement)statement, &evalue, long_message, big_buff);
+        return_object = Py_BuildValue("iis", 0, evalue, long_message);
+        free(long_message);
+        return return_object;
     }
-    return Py_BuildValue("iis", rc, evalue, message);
+    
+    return Py_BuildValue("iis", 0, evalue, message);
 }
 
 
@@ -1233,6 +1277,9 @@ static PyObject* mimerGetBlobData(PyObject* self, PyObject* args)
     }
 
     data = (char*) calloc(length, sizeof(char));
+    if(data == NULL) {
+        return(Py_BuildValue("i", 101));
+    }
     rc = MimerGetBlobData(&lobhandle, data, length);
     return_object = Py_BuildValue("iy#", rc, data,length);
     free(data);
@@ -1286,8 +1333,11 @@ static PyObject* mimerGetNclobData8(PyObject* self, PyObject* args)
         return Py_BuildValue("i", rc);
     }
 
-    data = (char*) calloc(length + lengthof_Terminating_NUL, sizeof(char));
-    rc = MimerGetNclobData8(&lobhandle, data, length + lengthof_Terminating_NUL);
+    data = (char*) calloc(4*length + lengthof_Terminating_NUL, sizeof(char));
+    if(data == NULL) {
+        return(Py_BuildValue("i", 101));
+    }
+    rc = MimerGetNclobData8(&lobhandle, data, 4*length + lengthof_Terminating_NUL);
     return_object = Py_BuildValue("is", rc, data);
     free(data);
     return return_object;
@@ -1372,6 +1422,9 @@ static PyObject* mimerGetBinary(PyObject* self, PyObject* args)
     if (rc > BUFLEN) {
         PyObject* return_object;
         char *bigvalue = malloc(rc);
+        if(bigvalue == NULL) {
+            return(Py_BuildValue("i", 101));
+        }
         rc = MimerGetBinary((MimerStatement)statement, parameter_number,
                             bigvalue, rc);
         return_object = Py_BuildValue("iy#", rc, bigvalue, rc);
@@ -1498,6 +1551,8 @@ static PyObject* mimerSetNull(PyObject* self, PyObject* args)
 */
 static PyMethodDef mimerapiMethods[] =
 {
+    {"mimerAPIVersion", (PyCFunction)mimerAPIVersion, METH_VARARGS,
+     "Get version number of the MimerAPI dynamic library."},
     {"mimerBeginSession8", (PyCFunction)mimerBeginSession8, METH_VARARGS,
      "Starts a session with the database."},
     {"mimerEndSession", (PyCFunction)mimerEndSession, METH_VARARGS,
