@@ -29,10 +29,17 @@ class TestCursorMethods(unittest.TestCase):
 ## Tests below
 ########################################################################
 
+    # Not working, will be fixed in the next version
+    @unittest.skip
+    def test_fetchall_ts(self):
+        with self.tstcon.cursor() as c:
+            c.execute("select 'a', cast('2020-09-17 11:21:51' as timestamp(2)) from system.onerow")
+            self.assertEqual(c.fetchall(), [('a', '2020-09-17 11:21:51.00')])
+
     def test_privilege(self):
         with self.tstcon.cursor() as c:
             with self.assertRaises(DatabaseError):
-                c.execute("create ident gurra as user using '123'")
+                c.execute("drop ident sysadm cascade")
 
     def test_createTable(self):
         with self.tstcon.cursor() as c:
@@ -259,6 +266,25 @@ class TestCursorMethods(unittest.TestCase):
                              [(1, 'продиктованной ангелом', 14.345),
                               (2, '安排他們參‱', 14.345)])
 
+    def test_insert_parametermarkers_goodchar(self):
+        with self.tstcon.cursor() as c:
+            c.execute("create table bobg14(c1 char(10)) in pybank");
+            self.tstcon.commit()
+            c.execute("insert into bobg14 values (?)", 'åååååååååå')
+            self.tstcon.commit()
+            c.execute("select * from bobg14")
+            self.assertEqual(c.fetchall(),
+                             [('åååååååååå',)])
+
+    def test_insert_parametermarkers_illchar(self):
+        with self.tstcon.cursor() as c:
+            c.execute("create table bobz14(c1 char(10)) in pybank");
+            self.tstcon.commit()
+            with self.assertRaises(ProgrammingError):
+                c.execute("insert into bobz14 values (?)", '安排')
+            with self.assertRaises(ProgrammingError):
+                c.execute("insert into bobz14 values (?)", '€')
+
     def test_insert_parametermarkers_too_long(self):
         with self.tstcon.cursor() as c:
             c.execute("create table bob15 (c1 NVARCHAR(10)) in pybank")
@@ -279,7 +305,7 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             c.execute("create table bob33 (c1 INTEGER, c2 NVARCHAR(10))"
                       " in pybank")
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.executemany("insert into bob33 values (:a, :b)",
                               ((3, 'pi', 14), (3)))
 
@@ -295,11 +321,11 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             c.execute("create table bob4 (c1 INTEGER) in pybank")
             c.executemany("insert into bob4 values (:a)", [(1,)])
-            with self.assertRaises(InterfaceError):
+            with self.assertRaises(ProgrammingError):
                 c.executemany("insert into bob4 values (:a)", [(1)])
-            with self.assertRaises(InterfaceError):
+            with self.assertRaises(ProgrammingError):
                 c.executemany("insert into bob4 values (:a)", (1))
-            with self.assertRaises(InterfaceError):
+            with self.assertRaises(ProgrammingError):
                 c.executemany("insert into bob4 values (:a)", [1])
 
     def test_executemany_one_tuple(self):
@@ -454,7 +480,7 @@ class TestCursorMethods(unittest.TestCase):
     def test_insert_wrong_type_parametermarkers(self):
         with self.tstcon.cursor() as c:
             c.execute("create table bob1337 (c1 INTEGER, c2 INTEGER) in pybank")
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert into bob1337  values (:a, :b)", (3, 3.14))
 
     def test_operate_after_closed(self):
@@ -536,8 +562,7 @@ class TestCursorMethods(unittest.TestCase):
     # Not 100% sure if InterfaceError is the correct error to be raised here
     def test_executemany_DDL(self):
         with self.tstcon.cursor() as c:
-            # Currently not throwing any exception
-            with self.assertRaises(InterfaceError):
+            with self.assertRaises(ProgrammingError):
                 b = c.executemany("create table bob6(c1 INTEGER) in pybank",
                                   (3))
 
@@ -545,7 +570,7 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             b = c.execute("create table bob16(c1 BIGINT) in pybank")
             big = pow(2, 100)
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert into bob16 values (?)", big)
 
     def test_insert_too_long(self):
@@ -568,14 +593,14 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             c.execute("create table jon31 (c1 INTEGER) in pybank")
             nvar = -2 ** 31 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon31 VALUES (?)", (nvar))
 
     def test_invalid_int32_insert_too_big(self):
         with self.tstcon.cursor() as c:
             c.execute("create table jon33 (c1 INTEGER) in pybank")
             var = 2 ** 31
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon33 VALUES (?)", (var))
 
     def test_valid_int64_insert(self):
@@ -590,7 +615,7 @@ class TestCursorMethods(unittest.TestCase):
             c.execute("create table intjon4 (c1 BIGINT, c2 BIGINT) in pybank")
             nvar = -2 ** 633
             var = 2 ** 63 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.executemany("insert INTO intjon4 VALUES (?,?)",
                               ((nvar, var), (nvar, var)))
 
@@ -598,14 +623,14 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             c.execute("create table jon63 (c1 BIGINT) in pybank")
             nvar = -2 ** 63 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon63 VALUES (?)", (nvar))
 
     def test_invalid_int64_insert_too_big(self):
         with self.tstcon.cursor() as c:
             c.execute("create table jon65 (c1 BIGINT) in pybank")
             var = 2 ** 63
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon65 VALUES (?)", (var))
 
     def test_valid_int16_insert(self):
@@ -619,14 +644,14 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             c.execute("create table jon15 (c1 SMALLINT) in pybank")
             nvar = -2 ** 15 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon15 VALUES (?)", (nvar))
 
     def test_invalid_int16_insert_too_small(self):
         with self.tstcon.cursor() as c:
             c.execute("create table jon17 (c1 SMALLINT) in pybank")
             nvar = 2 ** 15
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon17 VALUES (?)", (nvar))
 
     # &&&& Gives a Warning we dont catch atm
@@ -648,7 +673,7 @@ class TestCursorMethods(unittest.TestCase):
         f = self.tstcon.cursor()
         b.execute("create table jondd (c1 FLOAT) in pybank")
         var = 10 ** 309
-        with self.assertRaises(ProgrammingError):
+        with self.assertRaises(DataError):
             b.execute("insert INTO jondd VALUES (?)", (var))
 
         with self.assertRaises(ProgrammingError):
@@ -679,7 +704,7 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             c.execute("create table jonwas17 (c1 SMALLINT) in pybank")
             nvar = 2 ** 15
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jonwas17 VALUES (?)", (nvar))
             c.execute("insert INTO jonwas17 VALUES (?)", (5))
             self.assertEqual(c.messages, [])
@@ -688,9 +713,11 @@ class TestCursorMethods(unittest.TestCase):
         with self.tstcon.cursor() as c:
             c.execute("create table jonwas173 (c1 SMALLINT) in pybank")
             nvar = 2 ** 15
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jonwas173 VALUES (?)", (nvar))
-            self.assertEqual(c.messages[0][1], 'MicroAPI error -24010')
+            self.assertEqual(c.messages[0][1],
+                             (-24010,
+                              'Value was too large to fit in destination'))
 
     def test_None_is_returned(self):
         with self.tstcon.cursor() as c:
