@@ -278,11 +278,11 @@ class TestScrollCursorMethods(unittest.TestCase):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table bob4 (c1 INTEGER) in pybank")
             c.executemany("insert into bob4 values (:a)", [(1,)])
-            with self.assertRaises(InterfaceError):
+            with self.assertRaises(ProgrammingError):
                 c.executemany("insert into bob4 values (:a)", [(1)])
-            with self.assertRaises(InterfaceError):
+            with self.assertRaises(ProgrammingError):
                 c.executemany("insert into bob4 values (:a)", (1))
-            with self.assertRaises(InterfaceError):
+            with self.assertRaises(ProgrammingError):
                 c.executemany("insert into bob4 values (:a)", [1])
 
     def test_executemany_one_tuple(self):
@@ -427,7 +427,7 @@ class TestScrollCursorMethods(unittest.TestCase):
     def test_insert_wrong_type_parametermarkers(self):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table bob1337 (c1 INTEGER, c2 INTEGER)")
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert into bob1337  values (:a, :b)", (3, 3.14))
 
     def test_operate_after_closed(self):
@@ -507,7 +507,7 @@ class TestScrollCursorMethods(unittest.TestCase):
             self.tstcon.rollback()
 
     def test_executemany_DDL(self):
-        with self.assertRaises(InterfaceError):
+        with self.assertRaises(ProgrammingError):
             c = self.tstcon.executemany("create table bob6(c1 INTEGER)", (3))
         with self.tstcon.cursor(scrollable = True) as c:
             with self.assertRaises(ProgrammingError):
@@ -516,7 +516,7 @@ class TestScrollCursorMethods(unittest.TestCase):
     def test_insert_exceeded(self):
         c = self.tstcon.execute("create table bob16(c1 INTEGER) in pybank")
         big = pow(2,100)
-        with self.assertRaises(ProgrammingError):
+        with self.assertRaises(DataError):
             c.execute("insert into bob16 values (?)", big)
 
     def test_insert_too_long(self):
@@ -538,14 +538,14 @@ class TestScrollCursorMethods(unittest.TestCase):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jon31 (c1 INTEGER) in pybank")
             nvar = -2**31 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon31 VALUES (?)", (nvar))
 
     def test_invalid_int32_insert_too_big(self):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jon33 (c1 INTEGER) in pybank")
             var = 2**31
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon33 VALUES (?)", (var))
 
     def test_valid_int64_insert(self):
@@ -560,7 +560,7 @@ class TestScrollCursorMethods(unittest.TestCase):
             c.execute("create table intjon4 (c1 BIGINT, c2 BIGINT) in pybank")
             nvar = -2**633
             var = 2**63 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.executemany("insert INTO intjon4 VALUES (?,?)",
                               ((nvar,var),(nvar,var)))
 
@@ -568,14 +568,14 @@ class TestScrollCursorMethods(unittest.TestCase):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jon63 (c1 BIGINT) in pybank")
             nvar = -2**63 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon63 VALUES (?)", (nvar))
 
     def test_invalid_int64_insert_too_big(self):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jon65 (c1 BIGINT) in pybank")
             var = 2**63
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon65 VALUES (?)", (var))
 
     def test_valid_int16_insert(self):
@@ -589,14 +589,14 @@ class TestScrollCursorMethods(unittest.TestCase):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jon15 (c1 SMALLINT) in pybank")
             nvar = -2**15 - 1
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon15 VALUES (?)", (nvar))
 
     def test_invalid_int16_insert_too_small(self):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jon17 (c1 SMALLINT) in pybank")
             nvar = 2**15
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jon17 VALUES (?)", (nvar))
 
     # &&&& Gives a Warning we dont catch atm
@@ -627,7 +627,7 @@ class TestScrollCursorMethods(unittest.TestCase):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jonwas17 (c1 SMALLINT) in pybank")
             nvar = 2**15
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jonwas17 VALUES (?)", (nvar))
             c.execute("insert INTO jonwas17 VALUES (?)", (5))
             self.assertEqual(c.messages, [])
@@ -636,9 +636,11 @@ class TestScrollCursorMethods(unittest.TestCase):
         with self.tstcon.cursor(scrollable = True) as c:
             c.execute("create table jonwas173 (c1 SMALLINT) in pybank")
             nvar = 2**15
-            with self.assertRaises(ProgrammingError):
+            with self.assertRaises(DataError):
                 c.execute("insert INTO jonwas173 VALUES (?)", (nvar))
-            self.assertEqual(c.messages[0][1], 'MicroAPI error -24010')
+            self.assertEqual(c.messages[0][1],
+                             (-24010,
+                              'Value was too large to fit in destination'))
 
     def test_None_is_returned(self):
         with self.tstcon.cursor(scrollable = True) as c:
